{$A+,B-,D-,F-,G+,I+,K+,L-,N+,P+,Q-,R-,S+,T+,V-,W-,X+,Y-}
(******************************************************************************
**
**  BWCCDEL - Delphi encapsulation of BWCC controls.  Shows how to encapsulate
**            subclassing of controls from other DLLs.
**            Also supplies a Borland style dialogue (BWCCMessageBox) box which
**            expects the same parameters as the WinAPI MessageBox call.
**            I have added some interesting approaches here:
**              for Custom buttons, the three bitmaps are stored internally as a
**              three element array of bitmaps, but displayed in the object
**              inspector as three separate properties;
**            For the Custom buttons, I have allowed the caption to be set for
**              the purpose of accelerator key access, but the caption is never
**              displayed on the button.  The caption is NOT saved, but rather
**              an internal private variable is made to be persistent;
**            A neat trick for finding out what PAS unit a class is defined in,
**              see GETUNITNAME function.
**
**  History
**  =======
**
**  Version    Date   Description
**  2.10     05/07/97 Adding the new BorSign Type Into this BWCC Control Set.
**                    Renewed by Xiao Luo    sluox@wl.k12.in.us
**
**  1.10     30/06/97 Modified and sections rewritten by Andy Strong.
**                        e-mail: andy@nbaqsl.co.uk
**                    Added bwCustom BorBtn bitmap type.
**                    Fixed bug where pictures on BorBtns didn't always display.
**                    Fixed way BWCC version is generated.
**                    Fixed resource leak in BorBtn.
**                    Fixed button focus rectangle around BorBtn not clearing.
**                    Added Accelerator Key handling for all BorButton types.
**                    Fixed bug where focus rect did not show correctly around
**                      the Radio Buttons and CheckBoxes.
**                    Modified Caption handling for non bwWordOnly btns.
**                    Ensured BWCC load count only gets incremented by 1 not 2.
**                    Added dynamic load of library and functions.
**                    Downloaded from Delphi Super Page.
**  1.00     ??/??/?? Original by Xiao Luo
**
**  Original Message
**  ======== =======
**  You can use this program in any application you want without
**  notice the author of this program.  You may freely distrubute
**  this program as long as it's author is presented clearly as
**  the original author.  You may also change this program and reproduce
**  newer verison for this program as long as it's free.
**
**  Supported by DragonsoftMedia Inc. Xiao Luo (C) 1996,1997
**
*******************************************************************************)
unit Bwccdel;

interface

uses
  WinTypes, Messages, SysUtils, StdCtrls, Forms, Controls, Classes, Graphics,
  Menus;

const
 idhelp = 998;

 BBS_BITMAP = $8000;
 BBS_PARENTNOTIFY = $2000;
 BBS_OWNERDRAW = $1000;
 BBM_SETBITS = BM_SETSTYLE + 10;
 BBN_SETFOCUS = BN_DOUBLECLICKED + 10;
 BBN_SETFOCUSMOUSE = BN_DOUBLECLICKED + 11;
 BBN_GOTATAB = BN_DOUBLECLICKED + 12;
 BBN_GOTABTAB = BN_DOUBLECLICKED + 13;
 BBN_MOUSEENTER = BN_DOUBLECLICKED + 14;
 BBN_MOUSELEAVE = BN_DOUBLECLICKED + 14;

 BSS_GROUP = 1;
 BSS_HDIP = 2;
 BSS_VDIP = 3;
 BSS_HBUMP = 4;
 BSS_VBUMP = 5;
 BSS_RGROUP = 6;

 BSS_CAPTION = $8000;
 BSS_CTLCOLOR = $4000;
 BSS_NOPREFIX = $2000;
 BSS_LEFT = 0;
 BSS_CENTER = $0100;
 BSS_RIGHT = $0200;
 BSS_ALIGNMASK = $030;

 BWCCLIBRARYNAME = 'BWCC.DLL';

var
  LibHandle: THandle;

type
  Eloadlibraryerror = class(Exception);
  TDefStyle = (dsDefaultButton, dsPushButton);
  TSTATE = (tsHighlighted, tsNormal);
  TAccelString = string[2];
  TBWCCButtonKind = (bwWordOnly, bwOK, bwCancel, bwAbort, bwRetry, bwIgnore, bwYes, bwNo, bwHelp, bwCustom);

const
  { Base IDs in BWCC for standard buttons }
  BTNBMPBASES: array [0..2] of word = (1000, 3000, 5000);
  { Accelerator Captions for standard BorBtn buttons (same order as TBWCCBUTTONKIND) }
  BORBTNTEXTS: array[0..9] of TAccelstring = ('', '', '', '&A', '&R', '&I', '&Y', '&N', '', '');

type
  TBorBtn = class(TButtonControl)
  private
    CurrentKind: TDefStyle;
    CurrentState: TSTATE;
    CurrentBWCCKind: TBWCCButtonKind;
    FDefault: Boolean;
    FCancel: Boolean;
    FActive: Boolean;
    FReserved: Byte;
    FModalResult: TModalResult;
    FBitmaps: array[0..2] of HBitmap;       { Bitmaps to use for non WordOnly/Custom btns }
    FCustomBMPs: array[0..2] of TBitmap;    { Stored Bitmaps for Custom buttons }
    FCustomHandles: array[0..2] of HBitmap; { Array of HBitmaps for custom buttons }
    FCustomCaption: string;                 { 'Hidden' persistent Caption for custom btns }

    function GetKind:TBWCCButtonKind;
    function GetCaption: string;
    function CustomBMPsEmpty: boolean;

    procedure ReadCustCaption(Reader: TReader);
    procedure WriteCustCaption(Writer: TWriter);
    procedure SetBitmaps(Index: integer; Value: TBitmap);
    procedure SetCaption(Value: string);
    procedure SetKind(Value:TBWCCButtonKind);
    procedure SetDefault(Value: Boolean);
    procedure DeleteBMPS;
    procedure DefineProperties(Filer: TFiler); override;
    procedure CMDialogKey(var Message: TCMDialogKey); message CM_DIALOGKEY;
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CMFocusChanged(var Message: TCMFocusChanged); message CM_FOCUSCHANGED;
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
    procedure SetButtonStyle(ADefault: Boolean); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Click; override;
  published
    property BMPNormal: TBitmap index 0 read FCustomBMPs[0] write SetBitmaps;
    property BMPPressed: TBitmap index 1 read FCustomBMPs[1] write SetBitmaps;
    property BMPFocus: TBitmap index 2 read FCustomBMPs[2] write SetBitmaps;
    property Kind:TBWCCButtonKind read GetKind write SetKind;
    property Cancel: Boolean read FCancel write FCancel default False;
    property Caption: string read GetCaption write SetCaption;
    property Default: Boolean read FDefault write SetDefault default False;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ModalResult: TModalResult read FModalResult write FModalResult default 0;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop default True;
    property Visible;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnDBlClick;
  end;

  TBorRadio = class(TButtonControl)
  private
    FAlignment: TLeftRight;
    FChecked: Boolean;
    procedure SetAlignment(Value: TLeftRight);
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
  protected
    procedure SetChecked(Value: Boolean);
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Alignment: TLeftRight read FAlignment write SetAlignment default taRightJustify;
    property Caption;
    property Checked: Boolean read FChecked write SetChecked default False;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  TBorCheck = class(TButtonControl)
  private
    FAlignment: TLeftRight;
    FAllowGrayed: Boolean;
    FState: TCheckBoxState;
    FReserved: Byte;
    function GetChecked: Boolean;
    procedure SetAlignment(Value: TLeftRight);
    procedure SetChecked(Value: Boolean);
    procedure SetState(Value: TCheckBoxState);
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
  protected
    procedure Toggle; virtual;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property State: TCheckBoxState read FState write SetState default cbUnchecked;
    property Alignment: TLeftRight read FAlignment write SetAlignment default taRightJustify;
    property AllowGrayed: Boolean read FAllowGrayed write FAllowGrayed default False;
    property Caption;
    property Checked: Boolean read GetChecked write SetChecked stored False;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop default true;
    property Visible;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

 TShadeStyle = (ssGroup,ssRaisedgroup,ssHdip,ssVdip,ssHBump,ssVBump);
 TBorShade = class(TCustomControl)
  protected
    FStyle:TShadeStyle;
    function GetStyle:TShadeStyle;
    procedure SetStyle(value:TShadeStyle);
    procedure CreateParams(var Params: TCreateParams); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Style:TShadeStyle read GetStyle write SetStyle;
    property Ctl3D;
    property Align;
    property Caption;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

 TSignStyle=(Stop,Question,Exclamation,Information);
 TBorSign=Class(TGraphicControl)
  private
    property Canvas;
  protected
    FSignStyle:TSignStyle;
    procedure SetSignStyle(value:TSignStyle);
    procedure Paint;Override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Align;
    property SignStyle:TSignStyle read FSignStyle write SetSignStyle;
    property Color;
    property DragCursor;
    property DragMode;
    property ParentColor;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;



function  BWCCVersion: single;

procedure Register;

(*
**  Declare the function pointers for the calls into the BWCC DLL
*)
var
  BWCCRegister: procedure (hInst: word);
  BWCCmessagebox: function (WndParent: HWND; Txt,Caption: PChar; TextType: Word): Integer;
  BWCCIntlInit: function (language: word): boolean;
  BWCCIntlTerm: function : boolean;
  BWCCGetVersion: function : Longint;
  BWCCGetPattern: function : word;

implementation

uses
  WinProcs, TypInfo;

(*
**  Sanitised version of BWCC version number.
**  BWCCGetVersion Function returns a Longint.
**  The High Word returns locale, the low word returns the version number.
**  Locale 1 = US
**         2 = Japan.
**  This version function calculation gives:
**  LoWord(BWCCGetVersion) = 512;  Result := 2.00
**  LoWord(BWCCGetVersion) = 515;  Result := 2.03
*)

function BWCCVersion: single;
var
  HexVer: Word;
begin
  HexVer := LoWord(BWCCGetVersion);         { Get Version number, ignore Locale }
  Result := (HexVer shr 8) + ((HexVer mod 256) / 100);
end;

constructor TBorBtn.Create(AOwner: TComponent);
var
  i: integer;
begin
  inherited Create(AOwner);
  ControlStyle := [csSetCaption, csOpaque, csDoubleClicks];
  Width := 89;
  Height := 33;
  TabStop := True;
  Fillchar(FBitmaps, SizeOf(FBitmaps), 0);  { Clear all Button Bitmap holders }
  for i := 0 to 2 do
    FCustomBMPs[i] := TBitmap.Create;       { Create custom bitmap holders }
end;

(*
**  New Destructor to free up bitmap resources.
*)
destructor TBorBtn.Destroy;
var
  i: integer;
begin
  DeleteBMPs;                               { Release our resources (Bitmaps) }
  for i := 0 to 2 do
    FCustomBMPs[i].Free;                    { Release custom bitmap holders }
  inherited Destroy;
end;

(*
**  If button type not WordOnly, return caption for hard-wired list.
**  If button Custom, return our hidden persistent caption.
**  Otherwise return genuine caption.
*)
function TBorBtn.GetCaption: string;
begin
  case CurrentBWCCKind of
    bwCustom:
      Result := FCustomCaption;
    bwWordOnly:
      Result := inherited Caption;
    else
      Result := BORBTNTEXTS[Ord(CurrentBWCCKind)]
  end;
end;

(*
**  Simple check to see if all Custom Bitmaps are loaded.
*)
function TBorBtn.CustomBMPsEmpty: boolean;
begin
  Result := FCustomBMPs[0].Empty or FCustomBMPs[1].Empty or FCustomBMPs[2].Empty;
end;

(*
**  Sets one of the array of bitmaps to the passed value.  If type is set to
**  Custom, force a redisplay with the new BMPs.
*)
procedure TBorBtn.SetBitmaps(Index: integer; Value: TBitmap);
begin
  FCustomBMPs[Index].Assign(Value);
  if (CurrentBWCCKind = bwCustom) then
    SetKind(bwCustom);                      { Force Redisplay in case all BMPs loaded}
end;

(*
**  Only set caption if button type is WordOnly type.
*)
procedure TBorBtn.SetCaption(Value: string);
begin
  case CurrentBWCCKind of
    bwCustom:
      begin
        FCustomCaption := Value;            { Save 'Hidden' caption for later }
        inherited Caption := '';
      end;
    bwWordOnly:
      inherited Caption := Value;           { Save caption properly }
    else
      inherited Caption := '';              { Clear out any old text }
  end;
end;

(*
**  Delete the three bitmaps used by the Picture buttons, if the HBitmap handles
**  are non-zero.
*)
procedure TBorBtn.DeleteBMPs;
var
  i: integer;
begin
  for i := 0 to 2 do
    if FBitmaps[i] <> 0 then
    begin
      DeleteObject(FBitmaps[i]);
      FBitmaps[i] := 0;
    end;
end;

(*
**  This declaration allows our Persistent 'Hidden' object to become persistent.
**  Defines what methods are used to read/write, and whether or not to save the
**  data in the DFM unit.
*)
procedure TBorBtn.DefineProperties(Filer: TFiler);
begin
  Filer.DefineProperty('CustomCaption',     { Property name to write to DFM }
    ReadCustCaption, WriteCustCaption,      { Methods to Read and Write data }
    FCustomCaption <> '');                  { When to write the property }
  inherited DefineProperties(Filer);        { Declare all others of course! }
end;

(*
**  Reader method for our Persistent 'Hidden' caption property.  Called whilst
**  loading in data from the DFM unit, if a 'CustomCaption' property identified.
*)
procedure TBorBtn.ReadCustCaption(Reader: TReader);
begin
  FCustomCaption := Reader.ReadString;
end;

(*
**  Writer method for our Persistent 'Hidden' caption property.  Called whilst
**  saving data to the DFM unit, if a 'CustomCaption' property is not empty.
*)
procedure TBorBtn.WriteCustCaption(Writer: TWriter);
begin
  Writer.WriteString(FCustomCaption);
end;

procedure TBorBtn.Click;
var
  Form: TForm;
begin
  Form := GetParentForm(Self);
  if Form <> nil then
    Form.ModalResult := ModalResult;
  inherited Click;
end;

procedure TBorBtn.SetButtonStyle(ADefault: Boolean);
var
  Style: Word;
begin
  if HandleAllocated then
  begin
    if ADefault then
      Style := BS_DEFPUSHBUTTON
    else
      Style := BS_PUSHBUTTON;
    SendMessage(Handle, BM_SETSTYLE, Style, 1);
  end;
end;

procedure TBorBtn.SetDefault(Value: Boolean);
begin
  FDefault := Value;
  if HandleAllocated then
  with GetParentForm(Self) do               { Always send focus change message }
    Perform(CM_FOCUSCHANGED, 0, Longint(ActiveControl));
end;

procedure TBorBtn.CreateParams(var Params: TCreateParams);
const
  ButtonStyles: array[Boolean] of LongInt = (BS_PUSHBUTTON, BS_DEFPUSHBUTTON);
begin
  inherited CreateParams(Params);
  CreateSubClass(Params, 'borbtn');
  with Params do
    Style := Style or ButtonStyles[FDefault];
end;

(*
**  This will be called whenever the object needs to be created with a Windows
**  handle - even when RecreateWnd is called.
*)
procedure TBorBtn.CreateWnd;
begin
  inherited CreateWnd;
  FActive := FDefault;
  SetKind(CurrentBWCCKind);                 { Set BMPs for new button type }
end;

procedure TBorBtn.CNCommand(var Message: TWMCommand);
begin
  if Message.NotifyCode = BN_CLICKED then
    Click;
end;

procedure TBorBtn.CMDialogKey(var Message: TCMDialogKey);
begin
  with Message do
    if  (((CharCode = VK_RETURN) and FActive) or
      ((CharCode = VK_ESCAPE) and FCancel)) and
      (KeyDataToShiftState(Message.KeyData) = []) and CanFocus then
    begin
      Click;
      Result := 1;
    end else
      inherited;
end;

procedure TBorBtn.CMDialogChar(var Message: TCMDialogChar);
begin
  with Message do
    if IsAccel(CharCode, Caption) and CanFocus then
    begin
      Click;
      Result := 1;
    end else
      inherited;
end;

procedure TBorBtn.CMFocusChanged(var Message: TCMFocusChanged);
begin
  with Message do
    if Sender is TBorBtn then
      FActive := Sender = Self
    else
      FActive := FDefault;
  SetButtonStyle(FActive);
  inherited;
end;

function TBorBtn.GetKind: TBWCCbuttonKind;
begin
  Result := CurrentBWCCKind;
end;

(*
**  Re-written SetKind routine.  The 3 bitmaps needed for a BorBtn are now
**  stored in the object instance, and freed when the button type changes,
**  or the button is destroyed.
**  Streamlined code to use loop, and base ID offsets from BWCC.DLL.
*)
procedure TBorBtn.SetKind(Value:TBWCCbuttonKind);

  (*
  **  Loads bitmaps relevant to selected kind from BWCC.DLL.  If the type is set
  **  to bwCustom, the assumption is that there are no custom bitmaps, therefore
  **  set to blank bitmaps for displaying.
  *)
  procedure SetResBMPS;
  var
    i,
    Offset: integer;
  begin
    case CurrentBWCCKind of
      bwWordOnly, bwCustom:
        Offset := 999;                      { No captions }
      bwHelp:
        Offset := idHelp;                   { Special for help }
      else
        Offset := Ord(Value);               { Otherwise specified button }
    end;

    if not (CurrentBWCCKind in [bwWordOnly, bwCustom]) then
      inherited Caption := '';              { Clear old text }

    for i := 0 to 2 do
      FBitmaps[i] := LoadBitmap(LibHandle, MakeIntResource(BTNBMPBASES[i] + Offset));
    Perform(BBM_SETBITS, 0, longint(@FBitmaps));
  end;

var
  i: integer;
begin
  CurrentBWCCKind := Value;                 { Set new kind }
  DeleteBMPs;                               { Delete old BMPs }

  if (CurrentBWCCKind <> bwCustom) or       { Set to Custom with no bitmaps setup? }
    CustomBMPsEmpty then                    { (show blank bitmaps) }
    SetResBMPs                              { Load normal BMPS }
  else
  begin
    for i := 0 to 2 do                      { Use custom bitmaps }
      FCustomHandles[i] :=
        FCustomBMPS[i].Handle;              { Copy handles across to working hBitmaps }
    Perform(BBM_SETBITS, 0, longint(@FCustomHandles));
  end;
  Refresh;                                  { Update button display }
end;

constructor TBorRadio.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width := 113;
  Height := 20;                             { Was 17 - needs to be a little higher }
  ControlStyle := [csSetCaption, csDoubleClicks];
  FAlignment := taRightJustify;
end;

procedure TBorRadio.SetAlignment(Value: TLeftRight);
begin
  if FAlignment <> Value then
  begin
    FAlignment := Value;
    RecreateWnd;
  end;
end;

procedure TBorRadio.SetChecked(Value: Boolean);

procedure TurnSiblingsOff;
  var
    I: Integer;
    Sibling: TControl;
  begin
    if Parent <> nil then
      with Parent do
        for I := 0 to ControlCount - 1 do
        begin
          Sibling := Controls[I];
          if (Sibling <> Self) and (Sibling is TBorRadio) then
            TBorRadio(Sibling).SetChecked(False);
        end;
  end;

begin
  if FChecked <> Value then
  begin
    FChecked := Value;
    TabStop := Value;
    if HandleAllocated then
      SendMessage(Handle, BM_SETCHECK, Cardinal(Checked), 0);
    if Value then
    begin
      TurnSiblingsOff;
      Click;
    end;
  end;
end;

procedure TBorRadio.CreateParams(var Params: TCreateParams);
const
  Alignments: array[TLeftRight] of LongInt = (BS_LEFTTEXT, 0);
begin
  inherited CreateParams(Params);
  CreateSubClass(Params, 'borradio');
  with Params do
    Style := Style or BS_RADIOBUTTON or BBS_PARENTNOTIFY or Alignments[FAlignment];
end;

procedure TBorRadio.CreateWnd;
begin
  inherited CreateWnd;
  SendMessage(Handle, BM_SETCHECK, Cardinal(FChecked), 0);
end;

procedure TBorRadio.WMSetFocus(var Message: TWMSetFocus);
begin
  inherited;
end;

procedure TBorRadio.CMCtl3DChanged(var Message: TMessage);
begin
  RecreateWnd;
end;

procedure TBorRadio.CMDialogChar(var Message: TCMDialogChar);
begin
  with Message do
    if IsAccel(Message.CharCode, Caption) and CanFocus then
    begin
      SetFocus;
      Result := 1;
    end else
      inherited;
end;

procedure TBorRadio.CNCommand(var Message: TWMCommand);
begin
  case Message.NotifyCode of
    BN_CLICKED:
      SetChecked(True);
    BN_DOUBLECLICKED:
      DblClick;
  end;
end;

constructor TBorCheck.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width := 97;
  Height := 20;                             { Was 17 - needs to be a little higher }
  TabStop := True;
  ControlStyle := [csSetCaption, csDoubleClicks];
  FAlignment := taRightJustify;
  FState := cbUnchecked;
end;

procedure TBorCheck.Toggle;
begin
  case State of
    cbUnchecked:
      if AllowGrayed then
        State := cbGrayed
      else
        State := cbChecked;
    cbChecked:
      State := cbUnchecked;
    cbGrayed:
      State := cbChecked;
  end;
end;

function TBorCheck.GetChecked: Boolean;
begin
  Result := State = cbChecked;
end;

procedure TBorCheck.SetAlignment(Value: TLeftRight);
begin
  if FAlignment <> Value then
  begin
    FAlignment := Value;
    RecreateWnd;
  end;
end;

procedure TBorCheck.SetChecked(Value: Boolean);
begin
  if Value then State := cbChecked else State := cbUnchecked;
end;

procedure TBorCheck.SetState(Value: TCheckBoxState);
begin
  if FState <> Value then
  begin
    FState := Value;
    if HandleAllocated then
      SendMessage(Handle, BM_SETCHECK, Cardinal(FState), 0);
    Click;
  end;
end;

procedure TBorCheck.CreateParams(var Params: TCreateParams);
const
  Alignments: array[TLeftRight] of LongInt = (BS_LEFTTEXT, 0);
begin
  inherited CreateParams(Params);
  CreateSubClass(Params, 'borcheck');
  with Params do
    Style := Style or BS_3STATE or BBS_PARENTNOTIFY or Alignments[FAlignment];
end;

procedure TBorCheck.CreateWnd;
begin
  inherited CreateWnd;
  SendMessage(Handle, BM_SETCHECK, Cardinal(FState), 0);
end;

procedure TBorCheck.WMSetFocus(var Message: TWMSetFocus);
begin
  if Ctl3D and (Ctl3DBtnWndProc <> nil) then
    UpdateWindow(Handle);
  inherited;
end;

procedure TBorCheck.CMCtl3DChanged(var Message: TMessage);
begin
  RecreateWnd;
end;

procedure TBorCheck.CMDialogChar(var Message: TCMDialogChar);
begin
  with Message do
    if IsAccel(CharCode, Caption) and CanFocus then
    begin
      SetFocus;
      if Focused then
        Toggle;
      Result := 1;
    end else
      inherited;
end;

procedure TBorCheck.CNCommand(var Message: TWMCommand);
begin
  if Message.NotifyCode = BN_CLICKED then
    Toggle;
end;

constructor TBorshade.Create(Aowner:TComponent);
begin
  inherited create(AOWNER);
  Width := 50;
  Height := 50;
  FStyle := ssGroup;
end;

procedure Tborshade.CreateParams(var Params:
TCreateParams);
begin
  inherited CreateParams(Params);
  CreateSubClass(Params, 'borshade');
  with Params do
    Style := Style or Ord(FStyle) + 1 or BSS_Caption;
end;

function Tborshade.GetStyle:TShadeStyle;
begin
  result := Fstyle;
end;

procedure Tborshade.SetStyle(Value:TShadeStyle);
begin
  FStyle := Value;
  Recreatewnd;
end;

(* New Implementation of TBorSign *)


constructor TBorSign.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  FSignStyle:=Information;
  Height:=64;
  Width:=48;
  Invalidate;
end;

procedure TBorSign.Paint;
var a:TBitmap;
begin
  a:=Tbitmap.create;
  a.Handle:=loadbitmap(libHandle,MAKEINTRESOURCE(1900+Ord(FSignStyle)+1));
  Canvas.StretchDraw(ClientRect, A);
  a.free;
end;

procedure TBorSign.SetSignStyle(Value:TSignStyle);
begin
  FSignStyle:=Value;
  Invalidate;
end;



procedure Register;
  begin
    RegisterComponents('BWCC', [TBorBtn, TBorRadio, TBorCheck, TBorShade, TBorSign]);
  end;

procedure BWCCExitProc;far;
begin
  if LibHandle >= 32 then                   { Was library loaded }
    FreeLibrary(LibHandle);
  LibHandle := 0;
end;

(*
**  Initialisation of the BWCC library DLL.  Loads the DLL into Memory and
**  gets the function pointers for the required calls.  If either the load
**  of the library, or the acquisition of any of the function pointers fails
**  we will raise an Exception.
**  NOTE: the second parameter of the GETPROCADDRESS function calls looks
**  strange:  the function normally expects a PCHAR pointing to the function
**  name that is required;  however we cannot retrieve the pointer by this
**  method, we must use the INDEX of the function.  Casting the number to a
**  PCHAR is the only way to get the compiler to pass the syntax.
**  Indexes to function in BWCC found using Borlands IMPDEF program.
*)
procedure Initialise;

  (*
  **  A neat trick for determining the UNIT in which a class definition resides.
  *)
  function GetUnitName(const ClassType: TClass): string;
  begin
    Result := UpperCase(GetTypeData(PTypeInfo(ClassType.ClassInfo))^.UnitName);
  end;

  (*
  **  Local function to validate that an addres was retrieved.  If one wasn't
  **  we will not return!
  *)
  function ChkAddr(ProcAddr: TFarProc): TFarProc;
  begin
    Result := ProcAddr;                     { Setup return value }
    if not Assigned(ProcAddr) then          { Didn't find Procedure/function }
      raise ELoadLibraryError.Create('Function pointer not found in DLL!');
  end;

begin
  if LibHandle = 0 then                     { If already loaded - ignore }
  begin
    LibHandle :=
      WinProcs.LoadLibrary(BWCCLIBRARYNAME);{ Attempt to load DLL }
    if LibHandle < 32 then
      raise ELoadLibraryError.Create('Could NOT load Library: ' + BWCCLIBRARYNAME);
    {
      Load all the function pointers from the DLL into our function prototypes
    }
    try
      @BWCCRegister := ChkAddr(GetProcAddress(LibHandle, PChar(18)));
      @BWCCMessageBox := ChkAddr(GetProcAddress(LibHandle, PChar(9)));
      @BWCCIntlInit := ChkAddr(GetProcAddress(LibHandle, PChar(16)));
      @BWCCIntlTerm := ChkAddr(GetProcAddress(LibHandle, PChar(17)));
      @BWCCGetVersion := ChkAddr(GetProcAddress(LibHandle, PChar(11)));
      @BWCCGetPattern := ChkAddr(GetProcAddress(LibHandle, PChar(10)));
    except
      FreeLibrary(LibHandle);               { Release Loaded Library }
      LibHandle := 0;                       { Flag as NO LONGER loaded }
      raise;                                { Re-raise }
    end;
    if BWCCVersion < 2.00 then
      raise ELoadLibraryError.Create(
        'BWCC Version 2.00 or above required for the ''' + GetUnitName(TBorBtn) + ''' unit.');
  end;
end;

initialization
  LibHandle := 0;                           { Log as not yet loaded }
  Initialise;                               { Load Library and function pointers }
  AddExitProc(BWCCExitProc);                { Ensure library unloaded if loaded }
end.
